package klax;

import java.awt.*;
import java.util.TimerTask;

/**
 * @author Christian Grfe
 * 
 * Hier sind die Spielroutinen und -regeln abgelegt.
 * Das ist die Verwalltung des Spiels.
 * (Game-Code)
 */

public class JavaKlax_Engine extends TimerTask
{
    /** Die Kostanten fr die Tasten von Press() */
    public static final int K_LEFT = 0,
    						K_RIGHT= 1,
    						K_DROP_NORM = 2,
    						K_DROP_LIFO = 3,
    						K_PAUSE= 4;
    
    /** der Spielstatus und die passenden Konstanten */
    private int status;
    private static final int GAME_PAUSE=0,
    						 GAME_RUN=1,
    						 GAME_OVER =2;
    
    /** wieveil Ticks vergehen mssn, bis ein neuer Stein in den Trichter kommt */
    private static final byte next_stone_intervall=6;
    /** restliche Ticks bis zum neuen Stein im Trichter */
    private static byte next_stone;
    
    /** Die derzeitige maximale Anzahl an mglichen farben auf dem Feld<br>
     *  Aller 1500 Punkte steigt sie um 1<br>
     *  bis maximal 6 */
    private int max_farben;
    
    /** Der Spieler */
	JavaKlax_Player player;
	/** Der Auffangkorb */
	JavaKlax_Korb korb;
	/** Der Trichter wo die Stein herkommen */
	JavaKlax_Trichter trichter;
	
	/** das Frame in dem das Spiel luft */
	Frame parent_frame;
	
	/**
	 * Der Kostruktor erzeugt Trichter, Player und Korb<br>
	 * und ordnet sie auf dem Spielfeld an.<br>
	 * @param leben Anzahl von Leben des Spielers
	 */
	public JavaKlax_Engine(byte leben,Frame parent)
	{	
		parent_frame=parent;
		trichter=new JavaKlax_Trichter(23,40);
		
		player=new JavaKlax_Player(leben, (byte)2, trichter.getBottom()+(JavaKlax_Stein.hoehe>>1));
		
		korb=new JavaKlax_Korb(trichter.getBottom()+4*JavaKlax_Stein.hoehe+(JavaKlax_Stein.hoehe>>1));
		
		// setzt die Textfont
		parent.setFont(new Font("SansSerif",Font.BOLD,16));
		
		// Anfangswert fr MaxFarben setzen
		max_farben = 3;
		
		// Spiel starten
		status=GAME_RUN;
	}
	
	/**
	 * Zeichnet das Spiel
	 * @param g Der Grafikkontext auf den Gezeichnet wird
	 */
	public void draw(Graphics g)
	{
		// Punkte des Spielers
		g.setColor(Color.RED);
		g.drawString("Punkte: "+player.getPunkte(),10,40);
		
		// restlichen leben des Spielers
		for(int i=0;i<player.getLeben();i++)
			g.fillOval(350-20*i,27,16,16);
		
		switch(status)
		{
			case GAME_RUN:
				trichter.draw(g);
				player.draw(g);
				korb.draw(g);
				break;
			case GAME_PAUSE:
			    g.drawString("P A U S E !",parent_frame.getWidth()/2-40,parent_frame.getHeight()/2-10);
			    break;
			case GAME_OVER:
			    g.drawString("G A M E   O V E R",parent_frame.getWidth()/2-60,parent_frame.getHeight()/2-10);
			    g.drawString("zum Men mit ESC",parent_frame.getWidth()/2-70,parent_frame.getHeight()/2+10);
			    break;
		}
	}

	
	/**
	 * Ist fr die Eingabe ber Tastatur zustndig
	 * @param taste Taste die bettigt wurde (siehe Tasten_Kosntanten)<br>
	 * (K_LEFT, K_RIGHT, K_DROP_NORM, K_DROP_LIFO, K_PAUSE)
	 */
	public void press(int taste)
	{
	    switch (taste)
	    {
	    	// Spieler bewegen
	    	case K_LEFT: player.move(true); break;
	    	case K_RIGHT:player.move(false);break;
	    	
	    	// Stein vom Spieler weg
	    	case K_DROP_NORM:
	    	case K_DROP_LIFO:
	    	    JavaKlax_Stein stein;
	    	    int punkte;
	    	    // Stein vom Spieler hohlen und dem Korb bergeben
	    	    if(taste==K_DROP_NORM) stein=player.dropStein(false);
	    	    else stein=player.dropStein(true);
	    	    if(stein==null) break;
	    	    punkte=korb.getStein(stein);
	    	    // ausgabe von Korb auswerten
	    	    if(punkte==-1)	// der Stein konnte nicht abgelegt werden -> leben weg
	    	        if(player.killLeben()!=true) // -> Spieler tot?
	    	        {
	    	            status=GAME_OVER;
	    	            break;
	    	        }
	    	    // Punkte dem Spieler gutschreiben
	    	    player.addPunkte(punkte);
	    	    
	    	    // farbanzahl neu bestimmen
	    	    if(max_farben<8) max_farben=3+(int)Math.floor(player.getPunkte()/1500);
	    	    break;
	    	    
	    	// Spielpause einlegen
	    	case K_PAUSE: 
	    	    if (status==GAME_PAUSE) status=GAME_RUN;
	    	    else if(status==GAME_RUN) status=GAME_PAUSE;
	    	    break;
	    }
	}

	/** die Clock des Spiels */
	public void run()
	{
	    if(status==GAME_RUN)
	    {
			// dem trichter einen neuen stein bergeben
			if(next_stone==0)
			{
				next_stone=next_stone_intervall;
				trichter.getStein(new JavaKlax_Stein((byte)zufallszahl(0,max_farben),0,(byte)zufallszahl(0,4)));
			}
			else
			{
				next_stone--;
			}
			
			// den stein vom trichter hohlen
			JavaKlax_Stein stein=trichter.dropStein();
			if (stein!=null)
			{
				// stein an spieler bergeben
				if(player.getStein(stein)==false)
					if(player.killLeben()==false)
						status=GAME_OVER;
			}
			parent_frame.repaint();
	    }
	}
	
	/** 
	 * berechnet eine Zufallszahl
	 * @param von Anfangswert
	 * @param bis Endwert (inklusive)
	 * @return die Zufallszahl
	 */
	private int zufallszahl(int von, int bis)
	{
		return (int) (von + Math.floor(Math.random() * (bis-von+1)));
	}
}
