package klax;

import java.awt.Color;
import java.awt.Graphics;

/**
 * @author Christian Graefe
 * Enthlt den Trichter des Spiels
 * Er erhlt die Steine und lst sie durch sich durch flieen.
 */
public class JavaKlax_Trichter extends JavaKlax_GameObjekt
{
	/** Das Feld durch das sich die Steine bewegen<br>
	 *  Gre 5x40<br>
	 *  0,0 -> links oben
	 */
	private JavaKlax_Stein[][] trichter;
	
	/** die Unterkante des Trichters */
	private int y_bottom;
	
	/**
	 * Gibt den untersten Punkt vom Trichter zurck
	 * @return unterster Punkt des Trichters
	 */
	public int getBottom() { return y_bottom;}
	
	/**
	 * Der Konstruktor erzeugt ein Array(5x40) durch das sich die Steine bewegen.<br>
	 * Er legt die Eckpunkte des Trichters fest.<br>
	 * Er legt den linken Punkt der ersten Spalte fest.
	 * @param x -> x Position auf Spielfeld
	 * @param y -> y Position auf Spielfeld
	 */
	public JavaKlax_Trichter(int x, int y)
	{
		super(x,y);
		trichter = new JavaKlax_Stein[5][18];
		
		// untere Kannte des Trichters bestimmen
		y_bottom = y+(trichter[0].length)*JavaKlax_Stein.hoehe;
		
		// den linken Punkt der ersten Spallte bestimmen
		leftX=x;
		
	}
	
	/**
	 * bergibt dem Trichter einen Stein.<br>
	 * Er wird in die Spalte einsortiert, wie es in ihm gespeichert ist.
	 * @param stein der Stein der bergeben wird
	 */
	public void getStein(JavaKlax_Stein stein)
	{
		// Y des Steines auf y des Trichters setzen
		stein.setY(this.y);
		// Stein in den Trichter aufnehmen
		trichter[stein.getSpalte()][0]=stein;
	}
	
	/** rckt alle Steine um eine Zeile nach unten */
	private void nachruecken()
	{
		int i,j;
		
		// fr alle Spalten
		for(i=0;i<trichter.length;i++)
		{
			// alle eine Zeile nach unten rcken
			for(j=trichter[i].length-1;j>0;j--)
			{
				trichter[i][j]=trichter[i][j-1];
				if(trichter[i][j]!=null) trichter[i][j].setY(this.y+j*JavaKlax_Stein.hoehe);
			}
			// oberste Zeile 0
			trichter[i][0]=null;
		}
	}
	
	/**
	 * Hohlt einen Stein vom untersten Ende des Trichters ab<br>
	 * und rckt alles im Trichter nach
	 * @return ein Stein oder null wenn kein Stein da
	 */
	public JavaKlax_Stein dropStein()
	{
		// hier wird ein Stein zwischengespeichert
		JavaKlax_Stein stein=null;
		
		// die unterste zeile des trichters
		int zeile=trichter[0].length-1;	
		
		// in welcher Spalte steht ein Stein an
		for(int i=0;i<trichter.length;i++)
			if(trichter[i][zeile]!=null) stein=trichter[i][zeile]; 
		
		// im Trichter alles nachrcken lassen
		nachruecken();
		
		// den Stein zurckgeben
		return stein;
	}

	/** zeichnet den Trichter und seine Steine */
	public void draw(Graphics g)
	{
	    int i,j;
	    
	    // Farbe des Trichters
	    g.setColor(Color.RED);
	    
	    // Zwischenlinien
	    int x_pos;
	    for(i=0;i<=trichter.length;i++)
	    {
	        x_pos=leftX+i*(JavaKlax_Stein.breite+6);
	        g.drawLine(x_pos,y+JavaKlax_Stein.hoehe/2,x_pos,y_bottom-JavaKlax_Stein.hoehe/2);
	    }
		
		// Steine des Trichters zeichnen
		for(i=0;i<trichter.length;i++)
    		for(j=0;j<trichter[i].length;j++)
    			if(trichter[i][j]!=null)trichter[i][j].draw(g);
	}
	

}
